import * as _zag_js_anatomy from '@zag-js/anatomy';
import { RequiredBy, PropTypes, DirectionProperty, CommonProperties, NormalizeProps } from '@zag-js/types';
import * as _zag_js_core from '@zag-js/core';
import { StateMachine } from '@zag-js/core';

declare const anatomy: _zag_js_anatomy.AnatomyInstance<"root" | "label" | "tree" | "item" | "itemIndicator" | "itemText" | "branch" | "branchControl" | "branchTrigger" | "branchContent" | "branchText" | "branchIndicator">;

interface FocusChangeDetails {
    focusedValue: string | null;
}
interface ExpandedChangeDetails extends FocusChangeDetails {
    expandedValue: string[];
}
interface SelectionChangeDetails extends FocusChangeDetails {
    selectedValue: string[];
}
type ElementIds = Partial<{
    root: string;
    tree: string;
    label: string;
}>;
interface PublicContext extends DirectionProperty, CommonProperties {
    /**
     * The ids of the tree elements. Useful for composition.
     */
    ids?: ElementIds;
    /**
     * The id of the expanded nodes
     */
    expandedValue: string[];
    /**
     * The id of the selected nodes
     */
    selectedValue: string[];
    /**
     * The id of the focused node
     */
    focusedValue: string | null;
    /**
     * Whether the tree supports multiple selection
     * - "single": only one node can be selected
     * - "multiple": multiple nodes can be selected
     *
     * @default "single"
     */
    selectionMode: "single" | "multiple";
    /**
     * Called when the tree is opened or closed
     */
    onExpandedChange?: (details: ExpandedChangeDetails) => void;
    /**
     * Called when the selection changes
     */
    onSelectionChange?: (details: SelectionChangeDetails) => void;
    /**
     * Called when the focused node changes
     */
    onFocusChange?: (details: FocusChangeDetails) => void;
    /**
     * Whether clicking on a branch should open it or not
     * @default true
     */
    expandOnClick?: boolean;
    /**
     * Whether the tree supports typeahead search
     * @default true
     */
    typeahead?: boolean;
}
interface PrivateContext {
}
type ComputedContext = Readonly<{
    /**
     * @computed
     * Whether a typeahead search is ongoing
     */
    isTypingAhead: boolean;
    /**
     * @computed
     * Whether the tree supports multiple selection
     */
    isMultipleSelection: boolean;
}>;
type UserDefinedContext = RequiredBy<PublicContext, "id">;
interface MachineContext extends PublicContext, PrivateContext, ComputedContext {
}
interface MachineState {
    value: "idle";
}
type State = StateMachine.State<MachineContext, MachineState>;
type Send = StateMachine.Send<StateMachine.AnyEventObject>;
interface ItemProps {
    /**
     * The depth of the item or branch
     */
    depth: number;
    /**
     * The id of the item or branch
     */
    value: string;
    /**
     * Whether the item or branch is disabled
     */
    disabled?: boolean;
}
interface BranchProps extends ItemProps {
}
interface ItemState {
    /**
     * The value of the tree item
     */
    value: string;
    /**
     * Whether the tree item is disabled
     */
    disabled: boolean;
    /**
     * Whether the tree item is selected
     */
    selected: boolean;
    /**
     * Whether the tree item is focused
     */
    focused: boolean;
}
interface BranchState extends ItemState {
    /**
     * Whether the tree branch is expanded
     */
    expanded: boolean;
}
interface MachineApi<T extends PropTypes = PropTypes> {
    /**
     * The id of the expanded nodes
     */
    expandedValue: string[];
    /**
     * The id of the selected nodes
     */
    selectedValue: string[];
    /**
     * Function to expand nodes.
     * If no value is provided, all nodes will be expanded
     */
    expand(value?: string[]): void;
    /**
     * Function to collapse nodes
     * If no value is provided, all nodes will be collapsed
     */
    collapse(value?: string[]): void;
    /**
     * Function to select nodes
     * If no value is provided, all nodes will be selected
     */
    select(value?: string[]): void;
    /**
     * Function to deselect nodes
     * If no value is provided, all nodes will be deselected
     */
    deselect(value?: string[]): void;
    /**
     * Function to focus a branch node
     */
    focusBranch(value: string): void;
    /**
     * Function to focus an item node
     */
    focusItem(value: string): void;
    rootProps: T["element"];
    labelProps: T["element"];
    treeProps: T["element"];
    getItemState(props: ItemProps): ItemState;
    getItemProps(props: ItemProps): T["element"];
    getItemIndicatorProps(props: ItemProps): T["element"];
    getItemTextProps(props: ItemProps): T["element"];
    getBranchState(props: BranchProps): BranchState;
    getBranchProps(props: BranchProps): T["element"];
    getBranchIndicatorProps(props: BranchProps): T["element"];
    getBranchTriggerProps(props: BranchProps): T["element"];
    getBranchControlProps(props: BranchProps): T["element"];
    getBranchContentProps(props: BranchProps): T["element"];
    getBranchTextProps(props: BranchProps): T["element"];
}

declare function connect<T extends PropTypes>(state: State, send: Send, normalize: NormalizeProps<T>): MachineApi<T>;

declare function machine(userContext: UserDefinedContext): _zag_js_core.Machine<MachineContext, MachineState, _zag_js_core.StateMachine.AnyEventObject>;

export { type MachineApi as Api, type BranchProps, type BranchState, type UserDefinedContext as Context, type ElementIds, type ExpandedChangeDetails, type FocusChangeDetails, type ItemProps, type ItemState, type SelectionChangeDetails, anatomy, connect, machine };
